local EventHandler = FlexBar2.OO.Mixin
{
	"PreClick_RegisterCastingEvent", -- Register casting events so the button is notified from casting or channeling (this way, only clicked buttons are notified and memory is spared so not all butons do the check)
	"PostClick_UncheckButton", -- Uncheck the default checking of the button after it has been clicked
	"BAG_UPDATE_UpdateItems", -- Update grayed out stuff
	"UNIT_INVENTORY_CHANGED_UpdateGearChecks", -- Takes take of checking the buttons in the equipped color
	"ACTIONBAR_SLOT_CHANGED_UpdateTextures", -- Update button icons when an actionbar slot changed
	"PLAYER_ENTERING_WORLD_LoadButtonTextures", -- Update the button icons after all information is available to the UI
	"ACTIONBAR_UPDATE_COOLDOWN_SetSpinner", -- Set the cooldown spinner
	"SpellCast_Start", -- Triggered whenever a spellcast starts (whenever a button is clicked)
	"SpellCast_End", -- Triggered whenever a spellcast ends, be it on the SPELLCAST_FAILED or SPELLCAST_STOP event
	"Channeling_Start", -- Triggered when the player starts to channel a spell
	"Channeling_Stop", -- Triggered when the player stops channeling a spell
	"Buff_Update", -- Triggered when the player gains or loses a buff
	"Debuff_Gained", -- Triggered when the target gains a debuff
	"Debuff_Lost", -- Triggered when the target loses a debuff
	"Debuff_Update", -- Triggered when the player's target changes and that debuffs have to be updated
	"StartDragging", -- Start dragging the button, may be triggered for various reasons,
	"StopDragging", -- Stop dragging the button, may be triggered for various reasons
	"DragActionStart",
	"DragActionStop",
}


function EventHandler:Activate()
	self:RegisterScript("PreClick", "PreClick_RegisterCastingEvent");
	self:RegisterScript("PostClick", "PostClick_UncheckButton");
	self:RegisterScript("OnReceiveDrag", "DragActionStop");
	self:RegisterEvent("BAG_UPDATE", "BAG_UPDATE_UpdateItems");
	self:RegisterEvent("ACTIONBAR_SLOT_CHANGED", "Buff_Update");
	self:RegisterEvent("ACTIONBAR_SLOT_CHANGED", "ACTIONBAR_SLOT_CHANGED_UpdateTextures");
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "PLAYER_ENTERING_WORLD_LoadButtonTextures");
	self:RegisterEvent("SpecialEvents_UnitDebuffGained", "Debuff_Gained");
	self:RegisterEvent("SpecialEvents_UnitDebuffLost", "Debuff_Lost");
	self:RegisterEvent("SpecialEvents_AuraTargetChanged", "Debuff_Update");
	self:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN", "ACTIONBAR_UPDATE_COOLDOWN_SetSpinner");
	if(type(self.Settings) == "table" and type(self.Settings.Attributes) == "table" and self.Settings.Attributes.type == "action") then -- Trigger some functions to have buffs & worn gearn checked correctly
		self:Buff_Update(nil, self.Settings.Attributes.action);
		self:UNIT_INVENTORY_CHANGED_UpdateGearChecks(nil, "player");
	end
end


function EventHandler:DragActionStart()
end


function EventHandler:PLAYER_ENTERING_WORLD_LoadButtonTextures(Addon)
	self:LoadCurrentTexture();
end

function EventHandler:PreClick_RegisterCastingEvent()
	self:RegisterEvent("UNIT_SPELLCAST_START", "SpellCast_Start");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START","Channeling_Start");
end

function EventHandler:PostClick_UncheckButton()
	if(self.Check[1] ~= nil) then
		self.Frame:SetChecked(true);
	else
		self.Frame:SetChecked(false);
	end
end

function EventHandler:BAG_UPDATE_UpdateItems(Addon)
	if(type(self.Settings) == "table" and type(self.Settings.Attributes) == "table" and self.Settings.Attributes.type == "action"
	and IsConsumableAction(self.Settings.Attributes.action) and GetActionCount(self.Settings.Attributes.action) == 0) then
		self.Frame.Icon:SetVertexColor(0.5,0.5,0.5);
	else
		self.Frame.Icon:SetVertexColor(1,1,1);
	end
end


function EventHandler:UNIT_INVENTORY_CHANGED_UpdateGearChecks(Addon, Unit)
	if(Unit == "player" and type(self.Settings) == "table" and type(self.Settings.Attributes) == "table" and self.Settings.Attributes.type == "action"
	and HasAction(self.Settings.Attributes.action)
	and IsEquippedAction(self.Settings.Attributes.action) == 1) then
		self:SetChecked(true, "Equipped");
	else
		self:SetChecked(false, "Equipped");
	end
end

function EventHandler:ACTIONBAR_SLOT_CHANGED_UpdateTextures(Addon, Slot)
	-- Update actionbar slot texture if this button's slot corresponds with the changed slot(s)
	if(type(self.Settings) == "table" and type(self.Settings.Attributes) == "table" and self.Settings.Attributes.type == "action" and self.Settings.Attributes.action == Slot) then
		FlexBar2:TriggerEvent("ACTIONBAR_SLOT_CHANGED_UpdateTextures");
		self:LoadCurrentTexture();
	end
end

function EventHandler:ACTIONBAR_UPDATE_COOLDOWN_SetSpinner(Addon, MouseButton)
	if(self.Settings and self.Settings.Attributes and self.Settings.Attributes.type == "action") then
		local start, duration, enable = GetActionCooldown(self.Settings.Attributes.action);
		if(self.LastCooldown.start ~= start or self.LastCooldown.duration ~= duration) then
			self.LastCooldown.start = start;
			self.LastCooldown.duration = duration;
			if(FlexBar2.db.profile.Ticker == "global") then
				if(duration <= 1.5) then
					CooldownFrame_SetTimer(self.Frame.Cooldown, start, duration, enable);
				end
			elseif(FlexBar2.db.profile.Ticker == "noglobal") then
				if(duration > 1.5) then
					CooldownFrame_SetTimer(self.Frame.Cooldown, start, duration, enable);
				end
			elseif(FlexBar2.db.profile.Ticker == true) then
				CooldownFrame_SetTimer(self.Frame.Cooldown, start, duration, enable);
			end
		end
	end
end

function EventHandler:SpellCast_Start(Addon, Unit)
	if(Unit == "player" and self.Settings and self.Settings.Attributes and self.Settings.Attributes.type == "action" and IsCurrentAction(self.Settings.Attributes.action)) then
		FlexBar2:TriggerEvent("SpellCast_Start");
		self:SetChecked(true, "Casting");

		self:UnregisterEvent("UNIT_SPELLCAST_START", "SpellCast_Start");
		self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START","Channeling_Start");
		self:RegisterEvent("UNIT_SPELLCAST_STOP", "SpellCast_End");
	end
end

function EventHandler:SpellCast_End(Addon, Unit)
	if(Unit == "player" and self.Settings and self.Settings.Attributes and self.Settings.Attributes.type == "action") then
		FlexBar2:TriggerEvent("SpellCast_End");
		self:SetChecked(false, "Casting");

		self:UnregisterEvent("UNIT_SPELLCAST_STOP", "SpellCast_End");
	end
end

function EventHandler:Channeling_Start(Addon, Unit)
	if(Unit == "player" and self.Settings and self.Settings.Attributes and self.Settings.Attributes.type == "action" and IsCurrentAction(self.Settings.Attributes.action)) then
		FlexBar2:TriggerEvent("Channeling_Start");
		self:SetChecked(true, "Channeling");

		self:UnregisterEvent("UNIT_SPELLCAST_START", "SpellCast_Start");
		self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START","Channeling_Start");
		self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP", "Channeling_Stop");
	end
end

function EventHandler:Channeling_Stop(Addon, Unit)
	if(Unit == "player" and self.Settings and self.Settings.Attributes and self.Settings.Attributes.type == "action") then
		FlexBar2:TriggerEvent("Channeling_Stop");
		self:SetChecked(false, "Channeling");

		self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_STOP", "Channeling_Stop");
	end
end

function EventHandler:Buff_Update(Addon, Slot) -- We need to delay this slightly because buffs don't update immediately after the event triggers
	FlexBar2:ScheduleEvent(function()
		self.bufflist = FlexBar2:CopyTable(FlexBar2.Aura.vars.buffs);
		if(self.Settings and self.Settings.Attributes and self.Settings.Attributes.type == "action" and self.Settings.Attributes.action == Slot and HasAction(Slot)) then
			FlexBar2:TriggerEvent("Buff_Update");
			if(FlexBar2.Aura:UnitHasBuff("player", nil, nil, self:LoadCurrentTexture(true)) ~= nil) then
				self:SetChecked(true, "Buffed");
			else
				self:SetChecked(false, "Buffed");
			end
		end
	end, 0.2);
end

function EventHandler:Debuff_Gained(Addon, Unit, Name, Count, Type, Texture, Rank, Index)
	if(Unit == "target" and self.Settings and self.Settings.Attributes and self.Settings.Attributes.type == "action" and GetActionTexture(self.Settings.Attributes.action) == Texture) then
		self:SetChecked(true, "Debuffed");
	end
end

function EventHandler:Debuff_Lost(Addon, Unit, Name, Count, Applications, Type, Texture, Rank)
	if(Unit == "target" and self.Settings and self.Settings.Attributes and self.Settings.Attributes.type == "action" and GetActionTexture(self.Settings.Attributes.action) == Texture) then
		self:SetChecked(false, "Debuffed");
	end
end

function EventHandler:Debuff_Update(Addon)
	if(self.Settings and self.Settings.Attributes and self.Settings.Attributes.type == "action" and HasAction(self.Settings.Attributes.action) and FlexBar2.Aura:UnitHasDebuff("target", nil, nil, GetActionTexture(self.Settings.Attributes.action))) then
		self:SetChecked(true, "Debuffed");
	else
		self:SetChecked(false, "Debuffed");
	end
end

function EventHandler:StartDragging(Addon, MouseButton)
	if(MouseButton == "LeftButton") then
		self.Frame:StartMoving();
		self.IsMoving = true;
	end
end

function EventHandler:StopDragging(Addon)
	if(self.IsMoving == true) then
		self.Frame:StopMovingOrSizing();
		self.IsMoving = false;
		local _ ,_ ,_ ,X ,Y = self.Frame:GetPoint();
		self.Settings.X = X;
		self.Settings.Y = -Y;
	end
end

function EventHandler:DragActionStart(Addon)
	if(self.Settings and self.Settings.Attributes and self.Settings.Attributes.type == "action" and HasAction(self.Settings.Attributes.action)) then
		PickupAction(self.Settings.Attributes.action);
	end
end

function EventHandler:DragActionStop(Addon)
	if(self.Settings and self.Settings.Attributes and self.Settings.Attributes.type == "action") then
		PlaceAction(self.Settings.Attributes.action);
	end
end

FlexBar2:RegisterButtonClass(EventHandler);
